#!/usr/bin/perl
use strict;

use lib qw[/home/httpd/html/www/markup/html/whatpm/];

use Encode::Guess qw/euc-jp shiftjis 7bit-jis utf8/;
use Whatpm::HTML;

$| = 1;

my $current_category = 0;
our $target = shift;
our $code = sub {
  my ($entity, $file_name, $category) = @_;
  if ($current_category != $category) {
    print "\n\n";
    $current_category = $category;
  }
  eval {
    my $data = Encode::decode ('Guess', $entity->{body});
    tokenize ($data);
  };
};

require 'foreach.pl';

sub tokenize ($) {
  my %data;

  my $s = \($_[0]);
  my $p = Whatpm::HTML->new;
  my $i = 0;
  $p->{set_next_input_character} = sub {
    my $self = shift;
    $self->{next_input_character} = -1 and return if $i >= length $$s;
    $self->{next_input_character} = ord substr $$s, $i++, 1;
    
    if ($self->{next_input_character} == 0x000D) { # CR
      if ($i >= length $$s) {
        #
      } else {
        my $next_char = ord substr $$s, $i++, 1;
        if ($next_char == 0x000A) { # LF
          #
        } else {
          push @{$self->{char}}, $next_char;
        }
      }
      $self->{next_input_character} = 0x000A; # LF # MUST
    } elsif ($self->{next_input_character} > 0x10FFFF) {
      $self->{next_input_character} = 0xFFFD; # REPLACEMENT CHARACTER # MUST
    } elsif ($self->{next_input_character} == 0x0000) { # NULL
      $self->{next_input_character} = 0xFFFD; # REPLACEMENT CHARACTER # MUST
    }
  };
              
  $p->{parse_error} = sub { };
      
  $p->_initialize_tokenizer;

  my $start_tag_name = {};
  my $end_tag_name = {};
  my $attr;
  my $value;
  my $value2; # case insensitive

  while (1) {
    my $token = $p->_get_next_token;
    last if $token->{type} eq 'end-of-file';
    
    if ($token->{type} eq 'start tag') {
      if ({
           title => 1,
           textarea => 1,
          }->{$token->{tag_name}}) {
        $p->{content_model_flag} = 'RCDATA';
      } elsif ({
                style => 1,
                script => 1,
                xmp => 1,
                noframes => 1,
                noembed => 1,
                noscript => 1,
                iframe => 1,
               }->{$token->{tag_name}}) {
        $p->{content_model_flag} = 'CDATA';
      } elsif ($token->{tag_name} eq 'plaintext') {
        $p->{content_model_flag} = 'PLAINTEXT';
      }
      $p->{last_emitted_start_tag_name} = $token->{tag_name};
    }

    if ($token->{type} eq 'start tag') {
      if ({
           div => 1, span => 1,
           td => 1, th => 1,
           br => 1, 
           b => 1, i => 1, s => 1, strike => 1, u => 1, tt => 1,
           big => 1, small => 1,
           font => 1, basefont => 1,
           marquee => 1, blink => 1,
           hr => 1,
           center => 1,
           nobr => 1,
           layer => 1, ilayer => 1,
           spacer => 1,
           multicol => 1,
          }->{$token->{tag_name}}) {
        $data{presentational}++;
      } elsif ({
                img => 1, object => 1, applet => 1, embed => 1, iframe => 1,
                frame => 1, script => 1, canvas => 1, video => 1, audio => 1,
                bgsound => 1,
                input => 1, select => 1, textarea => 1, output => 1,
                datalist => 1, datagrid => 1,
                area => 1,
               }->{$token->{tag_name}}) {
        $data{object}++;
      } elsif ({
                h1 => 1, h2 => 1, h3 => 1, h4 => 1, h5 => 1, h6 => 1,
                li => 1,
                dt => 1, dd => 1,
                meta => 1, link => 1,
                acronym => 1, abbr => 1,
                cite => 1, ins => 1, del => 1, dfn => 1,
                strong => 1, em => 1,
                address => 1,
                caption => 1, thead => 1, tfoot => 1, col => 1, colgroup => 1,
               }->{$token->{tag_name}}) {
        $data{semantic}++;
      }
    } elsif ($token->{type} eq 'character') {
      $token->{data} =~ s/\s+//g;
      $data{text} += length $token->{data};
    }
  }

  my $x = 1000 / ($data{text} or 1);
  print $data{$_} * $x, "\t" for qw/presentational semantic object/;
  print "\n";
} # tokenize

=head1 AUTHOR

Wakaba <w@suika.fam.cx>.

=head1 LICENSE

Copyright 2007 Wakaba <w@suika.fam.cx>

This library is free software; you can redistribute it
and/or modify it under the same terms as Perl itself.

=cut

1;
## $Date: 2007/06/09 07:56:19 $

