#!/usr/bin/perl
use strict;

use Encode::Guess;

my $Version = {};
my $LastModified = {};
my $LM_duplicate = 0;
my $ContentType = {};
my $CT_duplicate = 0;
my $Server = {};
my $Server_duplicate = 0;

our $target = shift;
our $code = sub {
  my $entity = shift;
  $Version->{$entity->{protocol_version}}++;

  if ($entity->{field}->{'last-modified'}) {
    if (@{$entity->{field}->{'last-modified'}} > 1) {
      $LM_duplicate++;
    }
    my $lm = $entity->{field}->{'last-modified'}->[0];
    if ($lm =~ /^\s*(?>(?>Sun|Mon|Tue|Wed|Thu|Fri|Sat)\s*,\s*)?([0-9][0-9]?)\s*(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s*([0-9]{2,})/i) {
      $LastModified->{lc "$3$2"}++;
    } else {
      $LastModified->{$lm}++;
    }
  } else {
    $LastModified->{'(none)'}++;
  }

  if ($entity->{field}->{'content-type'}) {
    if (@{$entity->{field}->{'content-type'}} > 1) {
      $CT_duplicate++;
    }
    my $ct = $entity->{field}->{'content-type'}->[0];
    $ContentType->{$ct}++;
  } else {
    $ContentType->{'(none)'}++;
  }

  if ($entity->{field}->{'server'}) {
    if (@{$entity->{field}->{'server'}} > 1) {
      $Server_duplicate++;
    }
    my $server = $entity->{field}->{'server'}->[0];
    $server =~ s!/.*!!;
    $Server->{$server}++;
  } else {
    $Server->{'(none)'}++;
  }
};

require 'foreach.pl';

print "HTTP Version:\n";
for (sort {$a cmp $b} keys %{$Version}) {
  print $_, "\t", $Version->{$_}, "\n";
}
print "\n";

print "Server:\n";
for (sort {$a cmp $b} keys %$Server) {
  print $_, "\t", $Server->{$_}, "\n";
}
print "$Server_duplicate pages with Server duplication\n"
  if $Server_duplicate;
print "\n";

print "Last-Modified:\n";
for (sort {$a cmp $b} keys %$LastModified) {
  print $_, "\t", $LastModified->{$_}, "\n";
}
print "$LM_duplicate pages with Last-Modified duplication\n"
  if $LM_duplicate;
print "\n";

print "Content-Type:\n";
for (sort {$a cmp $b} keys %$ContentType) {
  print $_, "\t", $ContentType->{$_}, "\n";
}
print "$CT_duplicate pages with Content-Type duplication\n"
  if $CT_duplicate;
print "\n";

=head1 AUTHOR

Wakaba <w@suika.fam.cx>.

=head1 LICENSE

Copyright 2007 Wakaba <w@suika.fam.cx>

This library is free software; you can redistribute it
and/or modify it under the same terms as Perl itself.

=cut

1;
## $Date: 2007/06/02 12:12:28 $

