#!/usr/bin/perl
use strict;
use lib qw[/home/wakaba/work/manakai/lib];

require Message::DOM::DOMCore;
require Message::DOM::XMLParser;

my $dom = $Message::DOM::DOMImplementationRegistry->get_dom_implementation;
my $parser = $dom->create_ls_parser (1);
my $Error;

our $target = shift;
our $code = sub {
  my ($entity, $file_name) = @_;

  if ($entity->{body} =~ /^((?:(?!<html).)*?<!DOCTYPE[^>]*>)/is) {
    my $dt = $1;
    if ($dt =~ /xhtml/i or $dt =~ /<\?xml/i) {
      warn $file_name, "\n";
      parse_xml ($entity, $file_name);
    }
  }
};

$SIG{INT} = \&result;

require 'foreach.pl';

result ();
sub result {
  delete $SIG{INT};
  use Data::Dumper;
  $Data::Dumper::Sortkeys = 1;
  print Dumper $Error;
  exit;
}

sub parse_xml ($$) {
  my ($entity, $file_name) = @_;
  my $http_charset;
  if ($entity->{field}->{'content-type'}) {
    if ($entity->{field}->{'content-type'}->[0] =~ /charset\s*=\s*([^\s;]+)/i) {
      my $charset = $1;
      if ($charset =~ /euc/i) {
        $http_charset = 'euc-jp';
      } elsif ($charset =~ /shift/i or $charset =~ /sjis/i or 
               $charset =~ /ms932/i or $charset =~ /cp943/i or
               $charset =~ /windows-31j/i) {
        $http_charset = 'shift_jis';
      } elsif ($charset =~ /none/i or $charset =~ /unknown/i or
               $charset =~ /jp/i or $charset eq '0') {
        $http_charset = 'shift_jis' if $target =~ /keitai/i;
      } else {
        $http_charset = $charset;
      }
    } else {
      $http_charset = 'shift_jis' if $target =~ /keitai/;
    }
  }
  my $has_error;
  $Error->{$file_name} = [[$http_charset]];
  open my $s, '<', \($entity->{body});
  $parser->dom_config->set_parameter ('error-handler' => sub {
    my ($self, $err) = @_;
    my $loc = $err->location;
    push @{$Error->{$file_name}},
      [$loc->line_number, $loc->column_number, $err->type,
       $err->{'http://suika.fam.cx/~wakaba/archive/2004/dom/xml-parser#byte-sequence'}];
    #warn join "\t", $file_name, $err->stringify;
    $has_error = 1;
    return 1;
  });
  my $doc = $parser->parse
    ({byte_stream => $s, encoding => $http_charset});
  $Error->{$has_error ? '#error' : '#errorless'}++;
} # parse_xml

=head1 AUTHOR

Wakaba <w@suika.fam.cx>.

=head1 LICENSE

Copyright 2007 Wakaba <w@suika.fam.cx>

This library is free software; you can redistribute it
and/or modify it under the same terms as Perl itself.

=cut

1;
## $Date: 2007/07/21 05:26:48 $

